# Copyright (c) 2016. Mount Sinai School of Medicine
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function, division, absolute_import


from .allele_parse_error import AlleleParseError
from .parsing_helpers import parse_separator

def parse_mouse_allele_name(name):
    """Parses mouse MHc alleles such as H2-Kd, H-2-Db, H2-IAb.
    Returns pair of (gene, allele_code).
    """
    original = name
    if name.upper().startswith("H2"):
        name = name[2:]
    elif name.upper().startswith("H-2"):
        name = name[3:]
    _, name = parse_separator(name)

    # special logic for mouse alleles
    if name.upper().startswith("I"):
        # class II mouse allele
        if len(name) < 2:
            raise AlleleParseError("Incomplete mouse MHC allele: %s" % original)
        gene_name = name[:2]
        name = name[2:]
    else:
        # class I mouse allele
        if len(name) < 1:
            raise AlleleParseError("Incomplete mouse MHC allele: %s" % original)
        gene_name = name[0]
        name = name[1:]
    _, name = parse_separator(name)

    if len(name) != 1:
        raise AlleleParseError(
            "Malformed mouse MHC allele: %s, parse error at %s" % (
                original, name))
    allele = name[0]
    return gene_name.upper(), allele.lower()



from nose.tools import eq_

from mhcnames import (
    normalize_allele_name,
    compact_allele_name,
)

def test_human_class2():
    expected = "HLA-DRA1*01:01-DRB1*01:02"
    expected_compact = "DRB10102"
    for name in ["DRB1_0102",
                 "DRB101:02",
                 "HLA-DRB1_0102",
                 "DRB10102",
                 "DRB1*0102",
                 "HLA-DRB1*0102",
                 "HLA-DRB1*01:02"]:
        eq_(normalize_allele_name(name), expected)
        eq_(compact_allele_name(name), expected_compact)

def test_human_class2_alpha_beta():
    expected = "HLA-DPA1*01:05-DPB1*100:01"
    expected_compact = "DPA10105-DPB110001"
    for name in ["DPA10105-DPB110001",
                 "HLA-DPA1*01:05-DPB1*100:01",
                 "hla-dpa1*0105-dpb1*10001",
                 "dpa1*0105-dpb1*10001",
                 "HLA-DPA1*01:05/DPB1*100:01"]:
        eq_(normalize_allele_name(name), expected)
        eq_(compact_allele_name(name), expected_compact)



from nose.tools import eq_

from mhcnames import (
    normalize_allele_name,
    compact_allele_name,
    parse_allele_name,
    AlleleName,
)

def test_mouse_class2_alleles():
    # H2-IAb
    eq_(parse_allele_name("H2-IAb"),
        AlleleName("H-2", "IA", "", "b"))
    eq_(normalize_allele_name("H2-IAb"), "H-2-IAb")
    eq_(compact_allele_name("H2-IAb"), "IAb")

    # with hyphen in "H-2"
    eq_(parse_allele_name("H-2-IAb"),
        AlleleName("H-2", "IA", "", "b"))
    eq_(normalize_allele_name("H-2-IAb"), "H-2-IAb")
    eq_(compact_allele_name("H-2-IAb"), "IAb")



# Copyright (c) 2017. Mount Sinai School of Medicine
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function, division, absolute_import

from .allele_name import AlleleName
from .class2 import parse_classi_or_classii_allele_name

_normalized_allele_cache = {}
for omit_dra1 in [True, False]:
    _normalized_allele_cache[omit_dra1] = {}

_DRA1_0101 = AlleleName(
    species="HLA",
    gene="DRA1",
    allele_family="01",
    allele_code="01")

def normalize_allele_name(raw_allele, omit_dra1=False):
    """MHC alleles are named with a frustratingly loose system. It's not uncommon
    to see dozens of different forms for the same allele.

    Note: this function works with both class I and class II allele names (including
    alpha/beta pairs).

    For example, these all refer to the same MHC sequence:
        - HLA-A*02:01
        - HLA-A02:01
        - HLA-A:02:01
        - HLA-A0201
        - HLA-A00201

    Additionally, for human alleles, the species prefix is often omitted:
        - A*02:01
        - A*00201
        - A*0201
        - A02:01
        - A:02:01
        - A:002:01
        - A0201
        - A00201

    We might also encounter "6 digit" and "8 digit" MHC types (which specify
    variants that don't affect amino acid sequence), for our purposes these
    should be truncated to their "4-digit" forms:
        - A*02:01:01
        - A*02:01:01:01
    There are also suffixes which we're going to ignore:
        - HLA-A*02:01:01G

    And lastly, for human alleles, there are serotypes which we'll treat
    as approximately equal to a 4-digit type.
        - HLA-A2
        - A2

    These should all be normalized to:
        HLA-A*02:01
    """
    if raw_allele in _normalized_allele_cache[omit_dra1]:
        return _normalized_allele_cache[omit_dra1][raw_allele]
    parsed_alleles = parse_classi_or_classii_allele_name(raw_allele)
    species = parsed_alleles[0].species
    normalized_list = [species]
    # Optionally omit the alpha allele, e.g. for IEDB predictors.
    if omit_dra1 and len(parsed_alleles) == 2:
        alpha, beta = parsed_alleles
        # by convention the alpha allelle is omitted since it's assumed
        # to be DRA1*01:01
        if alpha == _DRA1_0101:
            parsed_alleles = [beta]
    for parsed_allele in parsed_alleles:
        if len(parsed_allele.allele_family) > 0:
            normalized_list.append("%s*%s:%s" % (
                parsed_allele.gene,
                parsed_allele.allele_family,
                parsed_allele.allele_code))
        else:
            # mice don't have allele families
            # e.g. H-2-Kd
            # species = H-2
            # gene = K
            # allele = d
            normalized_list.append("%s%s" % (
                parsed_allele.gene,
                parsed_allele.allele_code))
    normalized = "-".join(normalized_list)
    _normalized_allele_cache[omit_dra1][raw_allele] = normalized
    return normalized

def compact_allele_name(raw_allele):
    """
    Turn HLA-A*02:01 into A0201 or H-2-D-b into H-2Db or
    HLA-DPA1*01:05-DPB1*100:01 into DPA10105-DPB110001
    """
    parsed_alleles = parse_classi_or_classii_allele_name(raw_allele)
    normalized_list = []
    if len(parsed_alleles) == 2:
        alpha, beta = parsed_alleles
        # by convention the alpha allelle is omitted since it's assumed
        # to be DRA1*01:01
        if alpha == _DRA1_0101:
            parsed_alleles = [beta]

    for parsed_allele in parsed_alleles:
        if len(parsed_allele.allele_family) > 0:
            normalized_list.append("%s%s%s" % (
                parsed_allele.gene,
                parsed_allele.allele_family,
                parsed_allele.allele_code))
        else:
            # mice don't have allele families
            normalized_list.append("%s%s" % (
                parsed_allele.gene,
                parsed_allele.allele_code))
    return "-".join(normalized_list)



from nose.tools import eq_
from mhcnames import parse_allele_name, AlleleName

def test_dog_class2_allele():
    eq_(parse_allele_name("DLA-DQA1*00101"),
        AlleleName("DLA", "DQA1", "01", "01"))



from nose.tools import eq_
from mhcnames import (
    parse_allele_name,
    AlleleName,
    compact_allele_name,
    normalize_allele_name
)

def test_mouse_class1_alleles_H2_Kk():
    # H2-Kk
    eq_(parse_allele_name("H2-Kk"),
        AlleleName("H-2", "K", "", "k"))
    eq_(normalize_allele_name("H2-Kk"), "H-2-Kk")
    eq_(compact_allele_name("H-2-Kk"), "Kk")

    # with a hyphen in "H-2"
    eq_(parse_allele_name("H-2-Kk"),
        AlleleName("H-2", "K", "", "k"))
    eq_(normalize_allele_name("H-2-Kk"), "H-2-Kk")
    eq_(compact_allele_name("H-2-Kk"), "Kk")

def test_mouse_class1_alleles_H2_Db():
    # H2-Db
    eq_(parse_allele_name("H2-Db"),
        AlleleName("H-2", "D", "", "b"))
    eq_(normalize_allele_name("H2-Db"), "H-2-Db")
    eq_(compact_allele_name("H2-Db"), "Db")

    # with hyphen in "H-2"
    eq_(parse_allele_name("H-2-Db"),
        AlleleName("H-2", "D", "", "b"))
    eq_(normalize_allele_name("H-2-Db"), "H-2-Db")
    eq_(compact_allele_name("H-2-Db"), "Db")

def test_H2_Kd_without_seps():
    eq_(parse_allele_name("H2Kd"),
        AlleleName("H-2", "K", "", "d"))



from nose.tools import raises
from mhcnames import normalize_allele_name, AlleleParseError

@raises(AlleleParseError)
def test_extra_text_after_allele():
    normalize_allele_name("HLA-A*02:01 zipper")


from nose.tools import eq_
from mhcnames import normalize_allele_name, compact_allele_name

def test_macaque_alleles():
    allele_name = "Mamu-B*082:02"
    eq_(normalize_allele_name(allele_name), "Mamu-B*82:02")
    eq_(compact_allele_name(allele_name), "B8202")

    # expect 3rd zero in the family "007" to be trimmed in the normalized form
    # of this allele
    allele_name = "Mamu-B*007:02"
    eq_(normalize_allele_name(allele_name), "Mamu-B*07:02")
    eq_(compact_allele_name(allele_name), "B0702")



from nose.tools import eq_
from mhcnames import parse_allele_name, AlleleName

def test_sheep_class1_allele():
    eq_(parse_allele_name("Ovar-N*50001"),
        AlleleName("Ovar", "N", "500", "01"))

def test_sheep_class2_allele():
    eq_(parse_allele_name("Ovar-DRB1*0804"),
        AlleleName("Ovar", "DRB1", "08", "04"))



from .allele_name import (parse_allele_name, AlleleName)
from .normalization import (compact_allele_name, normalize_allele_name)
from .class2 import parse_classi_or_classii_allele_name
from .species import (
    species_name_to_prefixes,
    prefix_to_species_name,
)
from .allele_parse_error import AlleleParseError

__version__ = "0.3.9"

__all__ = [
    "AlleleName",
    "AlleleParseError",
    "compact_allele_name",
    "normalize_allele_name",
    "parse_allele_name",
    "parse_classi_or_classii_allele_name",
    "species_name_to_prefixes",
    "prefix_to_species_name"
]



# Copyright (c) 2016. Mount Sinai School of Medicine
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function, division, absolute_import

from collections import namedtuple

from .parsing_helpers import (
    parse_separator,
    parse_alphanum,
    parse_numbers,
    parse_letters
)
from .mouse import parse_mouse_allele_name
from .species import split_species_prefix
from .allele_parse_error import AlleleParseError

AlleleName = namedtuple("AlleleName", [
    "species",
    "gene",
    "allele_family",
    "allele_code"
])


def parse_allele_name(name, species_prefix=None):
    """Takes an allele name and splits it into four parts:
        1) species prefix
        2) gene name
        3) allele family
        4) allele code

    If species_prefix is provided, that is used instead of getting the species prefix from the name.
    (And in that case, a species prefix in the name will result in an error being raised.)

    For example, in all of the following inputs:
        "HLA-A*02:01"
        "A0201"
        "A00201"
    The result is a AlleleName object. Example:
        AlleleName(
            species="HLA",  # species prefix
            gene="A",    # gene name
            allele_family="02",   # allele family
            allele_code="01",   # allele code
        )

    The logic for other species mostly resembles the naming system for humans,
    except for mice, rats, and swine, which have archaic nomenclature.
    """
    original = name
    name = name.strip()

    if len(name) == 0:
        raise ValueError("Can't normalize empty MHC allele name")

    species_from_name, name = split_species_prefix(name)

    if species_prefix:
        if species_from_name:
            raise ValueError("If a species is passed in, we better not have another "
                             "species in the name itself.")
        species = species_prefix
    else:
        species = species_from_name

    if species in ("H-2", "H2"):
        gene, allele_code = parse_mouse_allele_name("H-2-" + name)
        # mice don't have allele families
        return AlleleName("H-2", gene, "", allele_code)

    if len(name) == 0:
        raise AlleleParseError("Incomplete MHC allele name: %s" % (original,))
    elif not species:
        # assume that a missing species name means we're dealing with a
        # human HLA allele
        if "-" in name:
            raise AlleleParseError("Can't parse allele name: %s" % original)
        species = "HLA"

    if name[0].upper() == "D":
        # MHC class II genes like "DQA1" need to be parsed with both
        # letters and numbers
        gene, name = parse_alphanum(name, 4)
        # TODO: make a list of known species/gene pairs, along with
        # gene synonyms. That should significantly imporve on this kind of
        # ad-hoc synonym handling.
        if gene == "DRA":
            gene = "DRA1"
        elif gene == "DRB":
            gene = "DRB1"
    elif name[0].isalpha():
        # if there are more separators to come, then assume the gene names
        # can have the form "DQA1"
        gene, name = parse_letters(name)
    elif name[0].isdigit():
        gene, name = parse_numbers(name)
    else:
        raise AlleleParseError(
            "Can't parse gene name from allele: %s" % original)

    if len(gene) == 0:
        raise AlleleParseError("No MHC gene name given in %s" % original)
    if len(name) == 0:
        raise AlleleParseError("Malformed MHC type %s" % original)

    gene = gene.upper()

    # skip initial separator
    sep, name = parse_separator(name)
    # If all that's left is e.g. "0201" then only parse the
    # first two digits as the family code. Also, human Class I alleles
    # seem to be exceptional in that they have only 2 digit allele
    # families but 3 digit allele codes
    # (other species like sheep have 3 digits followed by 2 digits)
    if len(name) == 4 or (species == "HLA" and gene in ("A", "B", "C")):
        family, name = parse_numbers(name, max_len=2)
    else:
        family, name = parse_numbers(name, max_len=3)

    sep, name = parse_separator(name)

    allele_code, rest_of_text = parse_numbers(name)

    rest_of_text = rest_of_text.strip()
    if len(rest_of_text) > 0:
        raise AlleleParseError("The suffix '%s' of '%s' was not parsed" % (
            original,
            rest_of_text))

    if len(family) == 1:
        family = "0" + family
    elif len(family) == 3 and family[0] == "0":
        family = family[1:]
    if len(allele_code) == 0:
        allele_code = "01"
    elif len(allele_code) == 1:
        # change HLA-A*2:01 into HLA-A*02:01
        allele_code = "0" + allele_code
    elif len(allele_code) == 3 and allele_code[0] == "0":
        # normalize HLA-A*02:001 into HLA-A*02:01
        allele_code = allele_code[1:]

    return AlleleName(species, gene, family, allele_code)



from nose.tools import eq_
from mhcnames import (
    normalize_allele_name,
    compact_allele_name,
)

hla_02_01_names = [
    "HLA-A*02:01",
    "HLA-A*0201",
    "A*02:01",
    "A*0201",
    "HLA-A02:01",
    # no punctuation
    "A0201",
    "HLA-A0201",
    "A0201",
    "A2",
    "A2:01",
    "HLA-A2",
    # lower case
    "hla-a*0201",
    "a*0201",
    "a*02:01",
    "a0201"
]

def test_hla_long_names():
    expected = "HLA-A*02:01"
    for name in hla_02_01_names:
        result = normalize_allele_name(name)
        eq_(result, expected)

def test_hla_short_names():
    expected = "A0201"
    for name in hla_02_01_names:
        result = compact_allele_name(name)
        eq_(result, expected)

def test_hla_with_3_digit_allele_code():
    # B*15:120
    eq_(normalize_allele_name("HLA-B*15:120"), "HLA-B*15:120")
    eq_(compact_allele_name("HLA-B*15:120"), "B15120")
    eq_(normalize_allele_name("B15120"), "HLA-B*15:120")
    eq_(compact_allele_name("B15120"), "B15120")

    # A*02*123
    eq_(normalize_allele_name("HLA-A*02:123"), "HLA-A*02:123")
    eq_(compact_allele_name("HLA-A*02:123"), "A02123")
    eq_(normalize_allele_name("A02123"), "HLA-A*02:123")
    eq_(compact_allele_name("A02123"), "A02123")



# Copyright (c) 2016. Mount Sinai School of Medicine
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function, division, absolute_import


def parse_substring(allele, pred, max_len=None):
    """
    Extract substring of letters for which predicate is True
    """
    result = ""
    pos = 0
    if max_len is None:
        max_len = len(allele)
    else:
        max_len = min(max_len, len(allele))
    while pos < max_len and pred(allele[pos]):
        result += allele[pos]
        pos += 1
    return result, allele[pos:]


SEPARATORS = {":", "*", "-"}

def parse_separator(allele, max_len=None):
    return parse_substring(allele, lambda c: c in SEPARATORS, max_len=max_len)

def parse_alphanum(allele, max_len=None):
    return parse_substring(allele, lambda c: c.isalnum(), max_len=max_len)

def parse_letters(allele, max_len=None):
    return parse_substring(allele, lambda c: c.isalpha(), max_len=max_len)

def parse_numbers(allele, max_len=None):
    return parse_substring(allele, lambda c: c.isdigit(), max_len=max_len)

def parse_not_numbers(allele, max_len=None):
    return parse_substring(allele, lambda c: not c.isdigit(), max_len=max_len)

def parse_until(allele, sep):
    return parse_substring(allele, lambda c: c != sep)



# Copyright (c) 2016. Mount Sinai School of Medicine
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function, division, absolute_import

from six import string_types

# copied from https://www.ebi.ac.uk/ipd/mhc/species.html
species_name_to_prefixes = dict(
    human="HLA",
    cattle="BoLA",
    bison="Bibi",
    dog="DLA",
    sheep=["Ovar", "OLA", "Ovca"],
    swine="SLA",
    mouse=["H2", "H-2"],
    rainbow_trout="Onmy",
    rat=["Rano", "Rara", "RT1"],
    salmon="Sasa",
    cat="FLA",
    horse=["ELA", "Eqca"],
    chimp=["Patr", "ChLA"],
    bonobo="Papa",
    white_handed_gibbon="Hyla",
    gorilla="Gogo",
    orangutan=["Popy", "OrLA"],
    blue_monkey="Cemi",
    de_brazzas_monkey="Cene",
    vervet_monkey="Chae",
    mantled_colobus="Cogu",
    black_mangabey="Loat",
    stump_tailed_macaque="Maar",
    crab_eating_macaque="Mafa",
    japanese_macaque="Mafu",
    rhesus_macaque=["Mamu", "RhLA"],
    pig_tailed_macaque="Mane",
    lion_tailed_macaque="Masi",
    drill="Male",
    mandrill="Masp",
    olive_baboon="Paan",
    yellow_baboon="Pacy",
    hamadryas_baboon="Paha",
    guinea_baboon="Papp",
    chacma_baboon="Paur",
    entelus_langur="Pren",
    gelada_baboon="Thge",
    owl_monkey=["Aoaz", "Aovo"],
    northern_night_owl_monkey=["Aona", "Aoni", "OmLA"],
    long_haired_spider_monkey="Atbe",
    brown_headed_spider_monkey="Atfu",
    marmoset=["Caja", "MaLA"],
    pygmy_marmoset="Cepy",
    dusk_titi_monkey="Camo",
    tufted_capuchin="Ceap",
    golden_lion_tamarin="Lero",
    white_faced_saki="Pipi",
    saddle_backed_tamarin="Safu",
    red_crested_tamarin="Sage",
    moustached_tamarin="Samy",
    cotton_top_tamarin="Saoe",
    squirrel_monkey="Sasc",
    lemur="Leca")

prefix_to_species_name = {}

_preferred_prefixes = []
_alternate_prefixes = []

for (species, prefixes) in species_name_to_prefixes.items():
    if isinstance(prefixes, string_types):
        prefixes = [prefixes]
    for i, prefix in enumerate(prefixes):
        prefix_to_species_name[prefix] = species
        if i == 0:
            _preferred_prefixes.append(prefix)
        else:
            _alternate_prefixes.append(prefix)

# list of all species specific prefixes in search order
_all_prefixes = _preferred_prefixes + _alternate_prefixes

def split_species_prefix(name, seps="-:_ "):
    """
    Splits off the species component of the allele name from the rest of it.

    Given "HLA-A*02:01", returns ("HLA", "A*02:01").
    """
    species = None
    name_upper = name.upper()
    name_len = len(name)
    for curr_prefix in _all_prefixes:
        n = len(curr_prefix)
        if name_len <= n:
            continue
        if name_upper.startswith(curr_prefix.upper()):
            species = curr_prefix
            name = name[n:].strip(seps)
            break
    return (species, name)



# Copyright (c) 2016. Mount Sinai School of Medicine
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function, division, absolute_import

from .species import split_species_prefix
from .allele_name import parse_allele_name, AlleleName
from .allele_parse_error import AlleleParseError

def parse_classi_or_classii_allele_name(name):
    """
    Handle different forms of both single and alpha-beta allele names.
    Alpha-beta alleles may look like:

    DPA10105-DPB110001
    HLA-DPA1*01:05-DPB1*100:01
    hla-dpa1*0105-dpb1*10001
    dpa1*0105-dpb1*10001
    HLA-DPA1*01:05/DPB1*100:01

    Other class II alleles may look like:

    DRB1_0102
    DRB101:02
    HLA-DRB1_0102
    """
    species, name = split_species_prefix(name)

    # Handle the case where alpha/beta pairs are separated with a /.
    name = name.replace("/", "-")

    # Ignored underscores, such as with DRB1_0102
    name = name.replace("_", "*")

    parts = name.split("-")
    if len(parts) > 2:
        raise AlleleParseError(
            "Allele has too many parts: %s" % name)
    if len(parts) == 1:
        parsed = parse_allele_name(name, species)
        if parsed.species == "HLA" and parsed.gene.startswith("DRB"):
            alpha = AlleleName(
                species="HLA",
                gene="DRA1",
                allele_family="01",
                allele_code="01")
            return (alpha, parsed)
        else:
            return (parsed,)
    else:
        alpha_string, beta_string = parts
        alpha = parse_allele_name(alpha_string)
        beta = parse_allele_name(beta_string)
        return (alpha, beta)



# Copyright (c) 2014. Mount Sinai School of Medicine
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import logging
import re

from setuptools import setup

readme_dir = os.path.dirname(__file__)
readme_filename = os.path.join(readme_dir, 'README.md')

try:
    with open(readme_filename, 'r') as f:
        readme = f.read()
except:
    logging.warn("Failed to load %s" % readme_filename)
    readme = ""

try:
    import pypandoc
    readme = pypandoc.convert(readme, to='rst', format='md')
    
except:
    logging.warn("Conversion of long_description from MD to RST failed")
    pass

with open('mhcnames/__init__.py', 'r') as f:
    version = re.search(
        r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]',
        f.read(),
        re.MULTILINE).group(1)

if not version:
    raise RuntimeError('Cannot find version information')

if __name__ == '__main__':
    setup(
        name='mhcnames',
        version=version,
        description="Python library for MHC nomenclature parsing",
        author="Alex Rubinsteyn",
        author_email="alex.rubinsteyn@mssm.edu",
        url="https://github.com/hammerlab/mhcnames",
        license="http://www.apache.org/licenses/LICENSE-2.0.html",
        classifiers=[
            'Development Status :: 3 - Alpha',
            'Environment :: Console',
            'Operating System :: OS Independent',
            'Intended Audience :: Science/Research',
            'License :: OSI Approved :: Apache Software License',
            'Programming Language :: Python',
            'Topic :: Scientific/Engineering :: Bio-Informatics',
        ],
        install_requires=['six>=1.9.0'],
        long_description=readme,
        packages=['mhcnames'],
    )



# TODO: test swine and rat alleles since those
# can be significantly different from other species we've implementeds

# Rat alleles:
# - RT1-Bb*u
# - RT1-Db1*a
# - RT1-DMa*a
# - RT1-9.5*f
# - RT1-M3-1*av1
